# Config Preview Deploy

A Drupal module that enables safe configuration deployment from preview/PR environments to production using configuration checkpoints and unified diff files. The module tracks when preview environments diverge from production and ensures only validated changes are deployed.

## AI Usage Disclosure

* AI has been used in crafting the majority of the code of the module, including the implementation of the main features, documentation, and examples.
* The module has been fully reviewed and tested by the contributor.

## Table of Contents

- [Overview](#overview)
- [Requirements](#requirements)
- [Installation & Setup](#installation--setup)
- [Configuration](#configuration)
- [Usage](#usage)
- [Security](#security)
- [Troubleshooting](#troubleshooting)
- [Development](#development)

## Overview

The module allows safe deployment of configuration changes from preview environments to production without requiring console access. It implements a dual-layer security model with OAuth 2.0 user authorization and hash-based request verification.

### Key Features

- **Configuration Tracking**: Tracks changes on production and verifies base-configuration checkpoint is recent before deployment
- **Rebase Workflow**: Helps setting up a "config rebase" workflow to re-base preview environment configuration on latest production changes
- **Authentication**: OAuth 2.0 authorization code flow + hash-based request integrity verification
- **Web UI & CLI**: Provides both Drupal UI and drush commands for seamless deployment

### Architecture

```
┌─────────────────────────────────────────┐     ┌─────────────────────────────────────────┐
│         PREVIEW ENVIRONMENT             │     │         PRODUCTION ENVIRONMENT          │
├─────────────────────────────────────────┤     ├─────────────────────────────────────────┤
│                                         │     │                                         │
│ • Creates checkpoint after prod sync    │     │ • OAuth authorize endpoint              │
│ • Tracks base timestamp                 │ ──▶ │ • Deploy endpoint (OAuth protected)     │
│ • Generates config diff                 │     │ • Creates checkpoint before import      │
│ • OAuth authorization flow              │ ◀── │ • Updates last_change timestamp         │
│ • Shows UI dashboard                    │     │ • Hash verification layer               │
│ • Verifies with production              │     │ • Logs all operations to watchdog       │
│ • Posts diff when valid                 │     │                                         │
│                                         │     │                                         │
└─────────────────────────────────────────┘     └─────────────────────────────────────────┘
```

### Target Use Case

This module is designed for teams using a CI/CD workflow with dynamic preview environments where:
- Configuration changes are made outside of Git-based deployment workflows
- Preview environments need to deploy config changes to production safely
- Multiple preview environments may exist simultaneously
- Site managers need explicit approval control over production deployments

## Requirements

- **Drupal**: 10.3+ or 11.x (requires CheckpointStorage API)
- **PHP**: 8.1+
- **Dependencies**: `simple_oauth`, `key` modules
- **System Tools**: GNU `patch` command for diff application
- **Hosting**: Any platform (Lagoon, Platform.sh, Pantheon, Acquia, etc.)

## Installation & Setup

### 1. Install Module

```bash
# Install the module (dependencies are installed automatically)
composer require drupal/config_preview_deploy

# Enable the module
drush en config_preview_deploy -y
```

### 2. Generate OAuth Keys

```bash
# Generate OAuth keys (required for simple_oauth)
drush simple-oauth:generate-keys /var/www/html/keys

# Or specify custom location
drush simple-oauth:generate-keys /app/oauth-keys
```

### 3. Configure Permissions

Navigate to `/admin/people/permissions` and assign:

- **Preview environments**:
  - `deploy config from preview` - For site managers deploying changes
  - `administer config preview deploy` - For administrators

- **Production environment**:
  - `accept config deployments` - For site managers accepting deployments

### 4. Production Environment Setup

On your **production** environment:

1. Enable the module and generate OAuth keys:
   ```bash
   drush en config_preview_deploy -y
   drush simple-oauth:generate-keys /var/www/html/keys
   ```

2. Configure production environment detection:

   The module automatically detects production environments using two methods:

   **Method 1: Environment Variable**
   ```bash
   # Force production mode via environment variable (highest priority)
   export DRUPAL_CONFIG_PREVIEW_DEPLOY_IS_PRODUCTION=1
   ```

   **Method 2: Automatic URL Matching**
   - Configure production URL in `/admin/config/development/config-preview-deploy/settings`
   - Module compares current site's base URL with configured production URL
   - Matches scheme, hostname, and port (ignores paths)
   - If URLs match, environment is detected as production

3. OAuth client secret configuration:

   The module automatically handles OAuth client secret in two ways:

   **Secure Method (Recommended)**:
   ```bash
   # Set environment variable with 64-character hex secret
   export DRUPAL_CONFIG_DEPLOY_SECRET="your-secure-64-char-hex-secret"
   ```

   **Automatic Fallback**:
   - If no environment variable is set, module auto-generates a secure secret
   - Secret stored via Key module as `config_deploy_secret` in configuration
   - Both preview and production environments must use the same secret
   - Both environments must share the same Drupal hash salt

### 5. Preview Environment Setup

1. Initialize preview with production data:
   ```bash
   # Sync database and config from production
   drush sql:sync @prod @self
   drush config:import
   ```

2. Initialize base checkpoint:
   ```bash
   # Creates initial checkpoint after sync
   drush config:preview-deploy:init
   ```

3. Register preview environment with production:

   Preview environments must be registered with production for OAuth security:

   ```bash
   # Register OAuth redirect URI for this preview environment
   drush @prod config:preview-deploy:register-environment https://preview-123.example.com
   ```

   ```bash
   # Remove old preview environment when no longer needed
   drush @prod cpd:register https://old-preview.example.com --remove
   ```

## Configuration

### Module Settings

Access settings at `/admin/config/development/config-preview-deploy/settings`:

- **Production URL**: URL of the production environment
- **Webhook Settings**: Optional rebase webhook configuration

### Environment Detection

**Environment Names**: The module automatically detects environment names from:
- `PHAPP_ENV` (drunomics/phapp-cli)
- `LAGOON_ENVIRONMENT`
- `PLATFORM_BRANCH`
- `PANTHEON_ENVIRONMENT`
- `ACQUIA_ENVIRONMENT`
- `CI_ENVIRONMENT_SLUG`
- Falls back to hostname or 'local'

**Production vs Preview Detection**: The module determines if an environment is production using:
- `DRUPAL_CONFIG_PREVIEW_DEPLOY_IS_PRODUCTION` environment variable (highest priority)
- Domain matching: compares current site's base URL with configured production URL
- If neither method identifies production, environment is treated as preview


## Usage

### Deployment Workflow

#### Web Interface

1. Navigate to `/admin/config/development/config-preview-deploy`
2. Review configuration changes shown in the dashboard
3. Click "Deploy to Production"
4. **OAuth Authorization**: Redirected to production for user consent
5. **Grant Permission**: Authorize deployment on production environment
6. **Return to Preview**: Automatic redirect with authorization
7. **Confirm Deployment**: Review and confirm deployment details
8. **Deploy**: Changes applied to production with checkpoint creation

#### Command Line

```bash
# Check current status
drush config:preview-deploy:status

# View configuration changes
drush config:preview-deploy:diff

# Save diff to file
drush config:preview-deploy:diff --output=/tmp/config-changes.patch

# Apply changes on production (requires manual file transfer)
drush @prod config:preview-deploy:apply /tmp/config-changes.patch --environment=preview
```

### Common Commands

```bash
# Initialize after production sync
drush cpd:init

# Show current status
drush cpd:status

# Generate and show diff
drush cpd:diff

# Save diff to file
drush cpd:diff --output=/tmp/config-changes.patch

# Trigger environment rebase (if webhook configured)
drush cpd:rebase
```

### Environment Rebase Workflow

When production configuration has changed and you need to update your preview environment:

```bash
# Save current changes before rebase
drush cpd:diff --output=/tmp/my-changes.patch

# Rebase: sync from production and re-initialize
drush sql:sync @prod @self
drush config:import
drush cpd:init

# Re-apply your changes
drush cpd:apply /tmp/my-changes.patch --environment=my-branch

# Deploy via web UI with OAuth authorization
```

### Rollback Support

Each deployment creates an automatic checkpoint. Install `drupal/config_checkpoint_ui` to manage rollbacks via `/admin/config/development/checkpoint` or use `drush config:checkpoint:restore CHECKPOINT_ID`.

## Security

The module implements a **dual-layer authentication system** for maximum security. For detailed implementation information, see [docs/ARCHITECTURE.md](docs/ARCHITECTURE.md).

### Security Overview

- **Layer 1**: OAuth 2.0 authorization code flow with explicit user consent
- **Layer 2**: Hash-based request integrity verification using shared hash salt
- **Production Detection**: Automatic environment detection with manual override support
- **Scope Isolation**: OAuth tokens restricted to deployment functionality only
- **Replay Protection**: Timestamp validation prevents request reuse

### Key Security Features

- **Explicit User Authorization**: Every deployment requires site manager approval on production
- **Token Isolation**: OAuth tokens cannot access other administrative functions
- **Environment Verification**: Hash verification ensures requests from legitimate sources
- **Audit Trail**: All operations logged for security monitoring

## Troubleshooting

### Common Issues

#### "No base checkpoint found"
```bash
# Solution: Initialize the preview environment
drush cpd:init
```

#### "OAuth authorization failed"
```bash
# Ensure preview environment is registered
drush @prod cpd:register https://your-preview-url.com

# Check OAuth keys exist
ls -la /var/www/html/keys/
```

#### "Invalid authentication" / Hash verification failed
```bash
# Ensure both environments have the same hash salt
# Check sites/default/settings.php for $settings['hash_salt']

# Verify production URL configuration
drush cget config_preview_deploy.settings production_url
```

#### "GNU patch tool not available"
```bash
# Install patch tool
apt install patch
# or
yum install patch
```

### Debug Commands

```bash
# Check module status
drush cpd:status

# Test diff generation
drush cpd:diff --output=/tmp/debug.patch

# Check recent log entries
drush watchdog:show --filter=config_preview_deploy

# Test production connectivity
curl -I https://your-production-site.com/api/config-preview-deploy/status
```

### Environment Verification

```bash
# Check environment detection
drush cpd:status

# Verify production environment detection
drush eval "echo \\Drupal::service('config_preview_deploy.preview_environment_access')->isProduction() ? 'Production' : 'Preview';"

# Check OAuth consumer configuration
drush cget consumers.consumer.config_preview_deploy
```

## Development

### Contributing

1. Follow Drupal coding standards
2. Add tests for new functionality
3. Use translatable strings for all user-facing text
4. Document new features in this README
5. Ensure all tests pass before submitting

### Testing

```bash
# Run module tests
./vendor/bin/phpunit web/modules/custom/config_preview_deploy/tests/

# Run specific test class
./vendor/bin/phpunit web/modules/custom/config_preview_deploy/tests/src/Kernel/ConfigVerifierTest.php

# Check coding standards
./vendor/bin/phpcs web/modules/custom/config_preview_deploy/
```

### Architecture Documentation

See [docs/ARCHITECTURE.md](docs/ARCHITECTURE.md) for detailed technical documentation of the authentication system and security model.
