# Config Preview Deploy - Authentication Architecture

## Overview

The Config Preview Deploy module implements a **dual-layer security model** for safe configuration deployment from preview environments to production. This architecture ensures that only authorized site managers from legitimate sources can deploy configuration changes.

## Security Layers

### Layer 1: OAuth Bearer Token (Site Manager Authorization)

**Purpose**: Verify that a legitimate site manager has explicitly authorized the deployment operation.

**Implementation**:
- Uses Drupal's `simple_oauth` module for standards-compliant OAuth 2.0 authentication
- Enforced at the route level using `_auth: ['oauth2']` in routing configuration
- Requires explicit user authorization flow for each deployment operation

**Security Rationale for Authorization Code Flow**:
- **Production-level permissions verification**: Ensures only users with actual production access can authorize deployments
- **Real-time permission validation**: User permissions are checked at the time of authorization, not based on potentially stale preview environment data
- **Explicit consent requirement**: Each deployment requires deliberate user action on the production environment
- **Preview environment isolation**: Even if preview environment user accounts are compromised or misconfigured, production authorization is still required

**Scope Strategy**:
- **Scope Name**: `config_preview_deploy`
- **Scope Type**: Dynamic OAuth scope entity (not static YAML configuration)
- **Permission-Based Granularity**: Maps to a single Drupal permission
- **Mapped Permission**: `'accept config deployments'`
- **Grant Types**: `authorization_code` for secure preview-to-production authorization
- **Security Benefit**: OAuth tokens are restricted to deployment functionality only

**Token Restriction**:
- Tokens with `config_preview_deploy` scope can only access routes requiring `'accept config deployments'` permission
- Even if the user has broader administrative permissions, the OAuth token cannot access other admin routes
- Provides strong security isolation following OAuth's principle of least privilege

### Layer 2: Hash Salt Verification (Source Legitimacy)

**Purpose**: Additional protection to verify that the deployment request originates from a legitimate preview environment.

**Implementation**:
- Hash-based verification using shared Drupal hash salt between environments
- Validated in controller methods after OAuth authentication passes
- Includes timestamp to prevent replay attacks

**Hash Calculation**:
- Method: `hash('sha256', $productionHost . $hashSalt . $timestamp)`
- Components: Production hostname + Drupal hash salt + current timestamp
- Validation: Production verifies hash matches expected value

## Authentication Flow

### 1. OAuth Authorization Phase

1. **User initiates deployment** from preview environment dashboard
2. **Redirect to production OAuth authorize endpoint** with authorization request
3. **User sees authorization screen** on production: "Allow config deployment access?"
4. **User grants permission** for the specific `config_preview_deploy` scope
5. **Production redirects back** to preview with authorization code
6. **Preview exchanges code for access token** using OAuth client credentials
7. **Access token stored** in preview environment session for deployment

### 2. Deployment Request Phase

1. **Preview generates deployment request** with configuration diff
2. **Hash salt verification** calculated using production hostname and timestamp
3. **API request sent** to production with OAuth bearer token and hash data
4. **Production validates OAuth token** automatically via route protection
5. **Production validates hash salt** in controller after OAuth passes
6. **Configuration deployed** if both authentication layers pass
7. **Response sent back** to preview with deployment results

## Route Protection Strategy

### Production Endpoints

All production configuration deployment endpoints are protected with:

- **OAuth Requirement**: `_auth: ['oauth2']` enforces bearer token validation
- **Permission Requirement**: `_permission: 'deploy config from preview'` matches OAuth scope
- **Dual Validation**: OAuth (automatic) + hash verification (controller-level)

### Preview Endpoints

Preview environment endpoints use standard Drupal permission checking:

- **Permission Requirement**: `_permission: 'deploy config from preview'` for deployment forms
- **Custom Access Control**: Environment detection to prevent production access to preview features

## Security Benefits

### OAuth Token Isolation

- **Scope Restriction**: Tokens cannot access other administrative functions
- **Time-Limited**: Tokens have configurable expiration times
- **Audit Trail**: OAuth provides logging of authorization grants

### Hash Salt Protection

- **Environment Verification**: Ensures requests come from legitimate preview environments
- **Replay Protection**: Timestamp validation prevents reuse of old requests
- **Shared Secret**: Requires knowledge of production hash salt

### Combined Security

- **Two-Factor Authentication**: Both OAuth approval AND hash verification required
- **Principle of Least Privilege**: OAuth tokens limited to minimal required permissions
- **Standards Compliance**: Uses OAuth 2.0 RFC specifications
- **Drupal Integration**: Works seamlessly with Drupal's permission system

## Configuration Requirements

### OAuth Setup

- **Consumer Auto-Creation**: OAuth consumer automatically created during module installation
  - **Client ID**: `config_preview_deploy`
  - **Scope Assignment**: Consumer must be assigned the `config_preview_deploy` scope
  - **Client Secret**: Stored in Key module as `config_deploy_secret`
- **Scope Configuration**: `config_preview_deploy` dynamic scope entity mapped to deployment permission
- **Grant Type**: Authorization code flow for secure user authorization
- **Client Credentials**: Managed by `simple_oauth` module
- **Redirect URI Registration**: Preview environments register via `drush cpd:register <uri>`
- **OAuth Keys**: Must be generated with `drush simple-oauth:generate-keys /var/www/html/keys`

### Environment Setup

- **Hash Salt Consistency**: Preview and production must share the same Drupal hash salt
- **Network Connectivity**: Preview environment must be able to reach production endpoints
- **Permission Configuration**: Users must have `'accept config deployments'` permission
- **Production URL**: Preview environment must be configured with correct production URL
- **Environment Detection**: Production environments detected via domain matching or `DRUPAL_CONFIG_PREVIEW_DEPLOY_IS_PRODUCTION` environment variable

## Error Handling

### OAuth Failures

- **Invalid Token**: Clear error messages when bearer token is missing or invalid
- **Expired Token**: Automatic re-authorization flow when tokens expire
- **Scope Mismatch**: Rejection when token lacks required scope
- **Network Issues**: Graceful handling of OAuth server connectivity problems

### Hash Verification Failures

- **Hash Mismatch**: Rejection when calculated hash doesn't match provided hash
- **Timestamp Issues**: Protection against replay attacks and clock skew
- **Missing Parameters**: Validation of required hash components
- **Environment Detection**: Clear messaging about hash salt configuration requirements

## Development and Testing

### Test Coverage Strategy

- **OAuth Flow Testing**: Complete authorization and token exchange flows
- **Token Restriction Testing**: Verification that tokens cannot access other routes
- **Hash Validation Testing**: Comprehensive hash calculation and verification testing
- **Integration Testing**: End-to-end deployment flows with full authentication
- **Security Testing**: Edge cases, invalid tokens, and attack scenario testing
