<?php

declare(strict_types=1);

namespace Drupal\config_preview_deploy;

use Drupal\Core\Config\Checkpoint\CheckpointListInterface;
use Drupal\Core\Config\Checkpoint\CheckpointStorage;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\KeyValueStore\KeyValueFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Component\Serialization\Yaml;
use SebastianBergmann\Diff\Differ;
use SebastianBergmann\Diff\Output\UnifiedDiffOutputBuilder;

/**
 * Handles diff generation from configuration checkpoints.
 *
 * 🔵 PREVIEW ENVIRONMENT ONLY
 * This class runs only on preview/staging to generate diffs for production.
 */
class CheckpointDiff {

  use StringTranslationTrait;

  /**
   * The active configuration storage.
   */
  protected StorageInterface $activeStorage;

  /**
   * The checkpoint storage.
   */
  protected CheckpointStorage $checkpointStorage;

  /**
   * The state service.
   */
  protected StateInterface $state;

  /**
   * Logger channel.
   */
  protected LoggerChannelInterface $logger;

  /**
   * Constructs a CheckpointDiff object.
   */
  public function __construct(
    StorageInterface $configStorage,
    KeyValueFactoryInterface $keyValueFactory,
    CheckpointListInterface $checkpoints,
    LoggerChannelFactoryInterface $loggerFactory,
    StateInterface $state,
  ) {
    $this->activeStorage = $configStorage;
    $this->checkpointStorage = new CheckpointStorage($configStorage, $checkpoints, $keyValueFactory);
    $this->state = $state;
    $this->logger = $loggerFactory->get('config_preview_deploy');
  }

  /**
   * Creates a checkpoint after initial config sync from production.
   *
   * @return string
   *   The checkpoint ID.
   */
  public function createBaseCheckpoint(): string {
    $checkpoint = $this->checkpointStorage->checkpoint(
      $this->t('Base checkpoint created - @date', [
        '@date' => date('Y-m-d H:i:s'),
      ])
    );

    $this->state->set('config_preview_deploy.base_version', [
      'timestamp' => time(),
      'checkpoint_id' => $checkpoint->id,
    ]);

    $this->logger->notice('Created base checkpoint: @id', ['@id' => $checkpoint->id]);
    return $checkpoint->id;
  }

  /**
   * Generates unified diff from base checkpoint to current state.
   *
   * @return string
   *   The unified diff string.
   *
   * @throws \RuntimeException
   *   When no base checkpoint exists.
   */
  public function generateDiff(): string {
    $baseVersion = $this->state->get('config_preview_deploy.base_version');
    if (!$baseVersion) {
      throw new \RuntimeException($this->t('No base checkpoint found. Run drush config:preview-deploy:init first.')->render());
    }

    $checkpointId = $baseVersion['checkpoint_id'];

    // Set checkpoint to read from.
    try {
      $this->checkpointStorage->setCheckpointToReadFrom($checkpointId);
    }
    catch (\Exception $e) {
      throw new \RuntimeException($this->t('Failed to set checkpoint @id: @message', [
        '@id' => $checkpointId,
        '@message' => $e->getMessage(),
      ])->render());
    }

    $diff = '';
    $changed = $this->getChangedConfigs();

    foreach ($changed as $name) {
      $originalExists = $this->checkpointStorage->exists($name);
      $currentExists = $this->activeStorage->exists($name);

      $original = $originalExists ? $this->checkpointStorage->read($name) : [];
      $current = $currentExists ? $this->activeStorage->read($name) : [];

      if ($original === $current) {
        continue;
      }

      // Determine proper headers based on file existence.
      if (!$originalExists && $currentExists) {
        // New file.
        $header = "--- /dev/null\n+++ b/$name.yml\n";
      }
      elseif ($originalExists && !$currentExists) {
        // Deleted file.
        $header = "--- a/$name.yml\n+++ /dev/null\n";
      }
      else {
        // Modified file.
        $header = "--- a/$name.yml\n+++ b/$name.yml\n";
      }

      // Handle YAML encoding properly for empty states.
      $originalYaml = empty($original) ? '' : Yaml::encode($original);
      $currentYaml = empty($current) ? '' : Yaml::encode($current);

      $builder = new UnifiedDiffOutputBuilder($header, TRUE);
      $differ = new Differ($builder);
      $configDiff = $differ->diff($originalYaml, $currentYaml);

      // Fix hunk header for new file creation (patch expects @@ -0,0 +1,N @@).
      if (!$originalExists && $currentExists) {
        $configDiff = preg_replace('/^@@ -1,0 (\+\d+,\d+) @@$/m', '@@ -0,0 $1 @@', $configDiff);
      }

      $diff .= $configDiff;
    }

    $this->logger->info('Generated diff from base checkpoint', [
      'changed_configs' => count($changed),
    ]);

    return $diff;
  }

  /**
   * Gets list of changed configurations since base checkpoint.
   *
   * @return array
   *   Array of changed configuration names.
   */
  public function getChangedConfigs(): array {
    $baseVersion = $this->state->get('config_preview_deploy.base_version');
    if (!$baseVersion) {
      return [];
    }

    $checkpointId = $baseVersion['checkpoint_id'];

    // Set checkpoint to read from.
    try {
      $this->checkpointStorage->setCheckpointToReadFrom($checkpointId);
    }
    catch (\Exception $e) {
      return [];
    }

    $current = $this->activeStorage->listAll();

    try {
      $original = $this->checkpointStorage->listAll();
    }
    catch (\Exception $e) {
      $original = [];
    }

    $changed = [];

    // Find added configs.
    $changed = array_merge($changed, array_diff($current, $original));

    // Find deleted configs.
    $changed = array_merge($changed, array_diff($original, $current));

    // Find actually modified configs by comparing content.
    $common = array_intersect($current, $original);
    foreach ($common as $name) {
      $currentData = $this->activeStorage->read($name) ?: [];
      $originalData = $this->checkpointStorage->read($name) ?: [];

      if ($currentData !== $originalData) {
        $changed[] = $name;
      }
    }

    return array_unique($changed);
  }

  /**
   * Checks if any configurations have changed since base checkpoint.
   *
   * @return bool
   *   TRUE if configurations have changed.
   */
  public function hasChanges(): bool {
    $changedConfigs = $this->getChangedConfigs();
    return !empty($changedConfigs);
  }

  /**
   * Gets the base version information.
   *
   * @return array|null
   *   Base version data or NULL if not set.
   */
  public function getBaseVersion(): ?array {
    return $this->state->get('config_preview_deploy.base_version');
  }

  /**
   * Gets the checkpoint storage.
   *
   * @return \Drupal\Core\Config\Checkpoint\CheckpointStorage
   *   The checkpoint storage.
   */
  public function getCheckpointStorage(): CheckpointStorage {
    return $this->checkpointStorage;
  }

}
