<?php

declare(strict_types=1);

namespace Drupal\config_preview_deploy\Controller;

use Drupal\config_preview_deploy\ProductionConfigDeployer;
use Drupal\config_preview_deploy\ConfigVerifier;
use Drupal\config_preview_deploy\HashVerification;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\File\FileSystemInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Production endpoints controller.
 */
class ProductionController extends ControllerBase {

  /**
   * The config verifier service.
   */
  protected ConfigVerifier $configVerifier;

  /**
   * The production config deployer service.
   */
  protected ProductionConfigDeployer $productionConfigDeployer;


  /**
   * The file system service.
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The hash verification service.
   *
   * @var \Drupal\config_preview_deploy\HashVerification
   */
  protected HashVerification $hashVerification;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->configVerifier = $container->get('config_preview_deploy.config_verifier');
    $instance->productionConfigDeployer = $container->get('config_preview_deploy.production_config_deployer');
    $instance->fileSystem = $container->get('file_system');
    $instance->hashVerification = $container->get('config_preview_deploy.hash_verification');
    return $instance;
  }

  /**
   * Deploys configuration diff endpoint.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with deployment result.
   */
  public function deploy(Request $request): JsonResponse {
    try {
      // Parse JSON request body.
      $content = $request->getContent();
      $data = json_decode($content, TRUE);

      if (!is_array($data)) {
        return new JsonResponse([
          'error' => $this->t('Invalid request format')->render(),
        ], 400);
      }

      // Validate required fields.
      $requiredFields = ['diff', 'environment', 'auth_hash', 'timestamp'];
      foreach ($requiredFields as $field) {
        if (!isset($data[$field])) {
          return new JsonResponse([
            'error' => $this->t('Missing required field: @field', ['@field' => $field])->render(),
          ], 400);
        }
      }

      $diff = $data['diff'];
      $environment = $data['environment'];
      $authHash = $data['auth_hash'];
      $timestamp = (int) $data['timestamp'];

      // Validate the diff before applying and keep temp storage for reuse.
      $validation = $this->productionConfigDeployer->validateDiff($diff);
      if (!$validation['valid']) {
        // Clean up temp storage on validation failure.
        if (isset($validation['temp_dir'])) {
          $this->fileSystem->deleteRecursive($validation['temp_dir']);
        }
        return new JsonResponse([
          'error' => $this->t('Invalid configuration diff: @errors', [
            '@errors' => implode(', ', $validation['errors']),
          ])->render(),
        ], 400);
      }

      try {
        // Deploy the configuration diff using pre-validated temp storage.
        $result = $this->productionConfigDeployer->deployDiff(
          $diff,
          $environment,
          $authHash,
          $timestamp,
          $validation['temp_storage'],
          $validation['temp_dir']
        );
      }
      finally {
        // Always clean up temp storage after deployment.
        if (isset($validation['temp_dir'])) {
          $this->fileSystem->deleteRecursive($validation['temp_dir']);
        }
      }

      // Update last change timestamp on successful deployment.
      $this->configVerifier->updateLastChange($environment);

      $this->getLogger('config_preview_deploy')->notice('Configuration deployed from @env with checkpoint @checkpoint on @prod_env', [
        '@env' => $environment,
        '@checkpoint' => $result['checkpoint_id'],
        '@prod_env' => $this->configVerifier->getEnvironmentName(),
      ]);

      return new JsonResponse($result);

    }
    catch (\RuntimeException $e) {
      // Authentication or deployment errors.
      return new JsonResponse([
        'error' => $e->getMessage(),
        'success' => FALSE,
      ], 403);
    }
    catch (\Exception $e) {
      $this->getLogger('config_preview_deploy')->error('Deployment endpoint error: @message', [
        '@message' => $e->getMessage(),
      ]);

      return new JsonResponse([
        'error' => $this->t('Internal server error')->render(),
        'success' => FALSE,
      ], 500);
    }
  }

  /**
   * Returns production configuration status with token authentication.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object with timestamp and token parameters.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with production configuration status.
   */
  public function getStatus(Request $request): JsonResponse {
    try {
      $timestamp = (int) $request->query->get('timestamp');
      $token = $request->query->get('token');

      // Validate token using production host + timestamp.
      if (!$this->hashVerification->verifyHash($token, $timestamp)) {
        return new JsonResponse(['error' => 'Invalid verification hash'], 403);
      }

      // Return production configuration status.
      $lastChange = $this->state()->get('config_preview_deploy.last_change', 0);
      $lastDeployedFrom = $this->state()->get('config_preview_deploy.last_deployed_from', '');

      $responseData = [
        'last_change' => $lastChange,
        'last_deployed_from' => $lastDeployedFrom,
        'environment' => $this->configVerifier->getEnvironmentName(),
        'timestamp' => time(),
      ];

      return new JsonResponse($responseData);

    }
    catch (\Exception $e) {
      $this->getLogger('config_preview_deploy')->error('Status endpoint error: @message', [
        '@message' => $e->getMessage(),
      ]);

      return new JsonResponse([
        'error' => $this->t('Internal server error')->render(),
      ], 500);
    }
  }

}
