<?php

declare(strict_types=1);

namespace Drupal\config_preview_deploy\Form;

use Drupal\config_preview_deploy\ConfigVerifier;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Rebase environment form.
 */
class RebaseForm extends FormBase {

  /**
   * The config verifier service.
   */
  protected ConfigVerifier $configVerifier;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static();
    $instance->configVerifier = $container->get('config_preview_deploy.config_verifier');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'config_preview_deploy_rebase_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['#title'] = $this->t('Rebase Environment');

    if (!$this->configVerifier->isRebaseAvailable()) {
      $form['not_configured'] = [
        '#markup' => '<div class="messages messages--error">' .
        $this->t('Rebase webhook is not configured. Please configure the webhook settings first.') .
        '</div>',
      ];

      $form['back'] = [
        '#type' => 'link',
        '#title' => $this->t('Back to Dashboard'),
        '#url' => Url::fromRoute('config_preview_deploy.dashboard'),
        '#attributes' => ['class' => ['button']],
      ];

      return $form;
    }

    $form['warning'] = [
      '#markup' => '<div class="messages messages--warning">' .
      $this->t('This will trigger a rebuild of your preview environment with the latest production configuration.') .
      '</div>',
    ];

    $form['description'] = [
      '#type' => 'details',
      '#title' => $this->t('What happens during rebase'),
      '#open' => TRUE,
    ];

    $form['description']['steps'] = [
      '#theme' => 'item_list',
      '#items' => [
        $this->t('Your current configuration changes will be preserved'),
        $this->t('A webhook will be triggered to rebuild your environment'),
        $this->t('The environment will sync with the latest production configuration'),
        $this->t('Your changes will be reapplied after the rebuild'),
        $this->t('A new base checkpoint will be created'),
      ],
    ];

    $form['confirmation'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('I understand that this will rebuild my environment'),
      '#required' => TRUE,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['rebase'] = [
      '#type' => 'submit',
      '#value' => $this->t('Rebase Environment'),
      '#button_type' => 'primary',
    ];

    $form['actions']['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => Url::fromRoute('config_preview_deploy.dashboard'),
      '#attributes' => ['class' => ['button']],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    try {
      $success = $this->configVerifier->triggerRebase();

      if ($success) {
        $this->messenger()->addStatus($this->t('Rebase webhook triggered successfully. Your environment will be rebuilt shortly.'));

        $this->messenger()->addStatus($this->t('Please wait for the rebuild to complete before making new configuration changes.'));
      }
      else {
        $this->messenger()->addWarning($this->t('Rebase webhook was triggered but may not have been successful. Check your hosting platform for rebuild status.'));
      }

      $form_state->setRedirect('config_preview_deploy.dashboard');

    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('Failed to trigger rebase: @message', ['@message' => $e->getMessage()]));
    }
  }

}
