<?php

declare(strict_types=1);

namespace Drupal\config_preview_deploy;

use Drupal\Core\Config\Checkpoint\CheckpointListInterface;
use Drupal\Core\Config\Checkpoint\CheckpointStorage;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Config\FileStorage;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Config\StorageComparer;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\KeyValueStore\KeyValueFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Component\Serialization\Yaml;

/**
 * Applies configuration diffs to the active configuration on production.
 */
class ProductionConfigDeployer {

  use StringTranslationTrait;

  /**
   * The active configuration storage.
   */
  protected StorageInterface $activeStorage;

  /**
   * The checkpoint storage.
   */
  protected CheckpointStorage $checkpointStorage;

  /**
   * The configuration factory.
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The configuration manager.
   */
  protected ConfigManagerInterface $configManager;

  /**
   * The production config importer service.
   */
  protected ProductionConfigImporterService $productionConfigImporterService;

  /**
   * The patch tool service.
   */
  protected PatchTool $patchTool;

  /**
   * Logger channel.
   */
  protected LoggerChannelInterface $logger;

  /**
   * The file system service.
   */
  protected FileSystemInterface $fileSystem;


  /**
   * The hash verification service.
   *
   * @var \Drupal\config_preview_deploy\HashVerification
   */
  protected HashVerification $hashVerification;

  /**
   * Constructs a ConfigDeployer object.
   */
  public function __construct(
    StorageInterface $configStorage,
    KeyValueFactoryInterface $keyValueFactory,
    CheckpointListInterface $checkpoints,
    ConfigFactoryInterface $configFactory,
    ConfigManagerInterface $configManager,
    ProductionConfigImporterService $productionConfigImporterService,
    PatchTool $patchTool,
    LoggerChannelFactoryInterface $loggerFactory,
    FileSystemInterface $fileSystem,
    HashVerification $hashVerification,
  ) {
    $this->activeStorage = $configStorage;
    $this->checkpointStorage = new CheckpointStorage($configStorage, $checkpoints, $keyValueFactory);
    $this->configFactory = $configFactory;
    $this->configManager = $configManager;
    $this->productionConfigImporterService = $productionConfigImporterService;
    $this->patchTool = $patchTool;
    $this->logger = $loggerFactory->get('config_preview_deploy');
    $this->fileSystem = $fileSystem;
    $this->hashVerification = $hashVerification;
  }

  /**
   * Applies a pre-validated configuration diff.
   *
   * @param string $diff
   *   The unified diff to apply.
   * @param string $environment
   *   The environment that is deploying.
   * @param string $authHash
   *   Authentication hash for security.
   * @param int $timestamp
   *   Timestamp used for authentication hash validation.
   * @param \Drupal\Core\Config\StorageInterface $tempStorage
   *   Pre-validated temp storage.
   * @param string $tempDir
   *   Pre-validated temp directory.
   *
   * @return array
   *   Deployment results with checkpoint ID and applied changes.
   *
   * @throws \RuntimeException
   *   When authentication fails or deployment fails.
   */
  public function deployDiff(string $diff, string $environment, string $authHash, int $timestamp, StorageInterface $tempStorage, string $tempDir): array {

    // Validate authentication including timestamp.
    if (!$this->hashVerification->verifyHash($authHash, $timestamp)) {
      $this->logger->warning('Invalid authentication in deployment request from @env', [
        '@env' => $environment,
      ]);
      throw new \RuntimeException($this->t('Invalid authentication')->render());
    }

    // Create checkpoint before applying changes.
    $checkpointLabel = $this->t('Import from @env - @date', [
      '@env' => $environment,
      '@date' => date('Y-m-d H:i:s'),
    ])->render();

    $checkpoint = $this->checkpointStorage->checkpoint($checkpointLabel);

    $this->logger->notice('Created checkpoint @id before import from @env', [
      '@id' => $checkpoint->id,
      '@env' => $environment,
    ]);

    try {
      // Apply the diff using pre-validated temp storage.
      $changes = $this->applyDiff($diff, $tempStorage, $tempDir);

      return [
        'success' => TRUE,
        'checkpoint_id' => $checkpoint->id,
        'changes' => $changes,
        'environment' => $environment,
      ];

    }
    catch (\Exception $e) {
      $this->logger->error('Configuration deployment failed from @environment: @error', [
        '@environment' => $environment,
        '@error' => $e->getMessage(),
      ]);

      throw new \RuntimeException($this->t('Configuration deployment failed: @error', ['@error' => $e->getMessage()])->render());
    }
  }

  /**
   * Applies a unified diff using pre-validated temp storage.
   *
   * @param string $diff
   *   The unified diff string.
   * @param \Drupal\Core\Config\StorageInterface $tempStorage
   *   Pre-validated temp storage.
   * @param string $tempDir
   *   Pre-validated temp directory.
   *
   * @return array
   *   Results of applying the diff, keyed by configuration name.
   */
  public function applyDiff(string $diff, StorageInterface $tempStorage, string $tempDir): array {
    // Use StorageComparer to detect and categorize changes.
    $storageComparer = new StorageComparer($tempStorage, $this->activeStorage);
    $storageComparer->createChangelist();

    // Check if there are actually changes to apply.
    if (!$storageComparer->hasChanges()) {
      return [];
    }

    // Validate site UUID to prevent cross-site imports.
    if (!$storageComparer->validateSiteUuid()) {
      throw new \Exception($this->t('Site UUIDs do not match, refusing to import configuration')->render());
    }

    // Create ConfigImporter with the storage comparer.
    $configImporter = $this->productionConfigImporterService->createImporter(
      $tempStorage,
      $this->activeStorage
    );

    // Validate import - this uses Drupal's built-in dependency validation.
    $configImporter->validate();

    // Import configurations with proper dependency ordering.
    $configImporter->import();

    // Build results array based on what was actually imported.
    $results = [];
    foreach ($storageComparer->getAllCollectionNames() as $collection) {
      foreach (['create', 'update', 'delete', 'rename'] as $operation) {
        foreach ($storageComparer->getChangelist($operation, $collection) as $configName) {
          $results[$configName] = $operation;
        }
      }
    }

    // Build detailed log message with file operations.
    $operationCounts = array_count_values($results);
    $operationSummary = [];
    $fileDetails = [];

    foreach (['create', 'update', 'delete', 'rename'] as $operation) {
      if (isset($operationCounts[$operation])) {
        $count = $operationCounts[$operation];
        $operationSummary[] = "$count $operation" . ($count > 1 ? 'd' : '');

        // Add file details for each operation.
        $filesForOperation = array_keys(array_filter($results, fn($op) => $op === $operation));
        foreach ($filesForOperation as $file) {
          $fileDetails[] = "$operation: $file";
        }
      }
    }

    $this->logger->info('Successfully imported @count configurations (@operations): @details', [
      '@count' => count($results),
      '@operations' => implode(', ', $operationSummary),
      '@details' => implode(', ', $fileDetails),
    ]);

    return $results;
  }

  /**
   * Creates a temporary directory with current config files as YAML.
   *
   * @return array
   *   Array containing [FileStorage $tempStorage, string $tempDir].
   *
   * @throws \Exception
   *   When creating temporary storage fails.
   */
  protected function createTempConfigDirectory(): array {
    // Create temporary directory for config files.
    $tempDir = $this->fileSystem->getTempDirectory() . '/config_preview_deploy_' . uniqid();
    if (!mkdir($tempDir, 0755, TRUE)) {
      throw new \Exception($this->t('Failed to create temporary directory for config import')->render());
    }

    $tempStorage = new FileStorage($tempDir);

    // Copy existing configurations to temp storage as YAML files.
    // This creates the structure that patch can work with.
    foreach ($this->activeStorage->listAll() as $configName) {
      $data = $this->activeStorage->read($configName);
      if ($data !== FALSE) {
        // Write as YAML file so patch tool can modify it.
        $yamlContent = Yaml::encode($data);
        file_put_contents($tempDir . '/' . $configName . '.yml', $yamlContent);
      }
    }

    return [$tempStorage, $tempDir];
  }

  /**
   * Cleans up temporary storage directory.
   *
   * @param string $directory
   *   The directory path to clean up.
   */
  protected function cleanupTempStorage(string $directory): void {
    if (is_dir($directory)) {
      // Use Drupal's file system service for proper cleanup.
      $this->fileSystem->deleteRecursive($directory);
    }
  }

  /**
   * Creates a checkpoint before applying configuration.
   *
   * @param string $label
   *   The checkpoint label.
   *
   * @return string
   *   The checkpoint ID.
   */
  public function createCheckpoint(string $label): string {
    $checkpoint = $this->checkpointStorage->checkpoint($label);

    $this->logger->notice('Created checkpoint @id with label "@label"', [
      '@id' => $checkpoint->id,
      '@label' => $label,
    ]);

    return $checkpoint->id;
  }

  /**
   * Validates that a diff can be safely applied and prepares temp storage.
   *
   * @param string $diff
   *   The diff to validate.
   *
   * @return array
   *   Validation result with 'valid', 'errors', 'config_count',
   *   'temp_storage', and 'temp_dir'.
   */
  public function validateDiff(string $diff): array {
    return $this->validateAndPrepareDiff($diff);
  }

  /**
   * Validates and applies a diff in one operation.
   *
   * For testing and convenience.
   *
   * @param string $diff
   *   The diff to validate and apply.
   *
   * @return array
   *   Results of applying the diff, keyed by configuration name.
   */
  public function validateAndApplyDiff(string $diff): array {
    $validation = $this->validateDiff($diff);

    if (!$validation['valid']) {
      // Clean up temp storage on validation failure.
      if ($validation['temp_dir']) {
        $this->cleanupTempStorage($validation['temp_dir']);
      }
      throw new \Exception($this->t('Configuration validation failed: @errors', ['@errors' => implode('; ', $validation['errors'])])->render());
    }

    try {
      return $this->applyDiff($diff, $validation['temp_storage'], $validation['temp_dir']);
    }
    finally {
      // Always clean up temp storage.
      if ($validation['temp_dir']) {
        $this->cleanupTempStorage($validation['temp_dir']);
      }
    }
  }

  /**
   * Validates a diff and prepares temp storage for import.
   *
   * @param string $diff
   *   The diff to validate and prepare.
   *
   * @return array
   *   Result array with validation info and prepared storage.
   */
  public function validateAndPrepareDiff(string $diff): array {
    $errors = [];
    $changeCount = 0;
    $tempStorage = NULL;
    $tempDir = NULL;

    // Check if patch tool is available.
    if (!$this->patchTool->isAvailable()) {
      $errors[] = $this->t('Patch tool not available. Please install GNU patch.')->render();
      return [
        'valid' => FALSE,
        'errors' => $errors,
        'config_count' => 0,
        'temp_storage' => NULL,
        'temp_dir' => NULL,
      ];
    }

    // Basic diff format validation.
    if (empty(trim($diff))) {
      $errors[] = $this->t('Empty diff provided')->render();
      return [
        'valid' => FALSE,
        'errors' => $errors,
        'config_count' => 0,
        'temp_storage' => NULL,
        'temp_dir' => NULL,
      ];
    }

    try {
      // Create temporary directory and apply patch to validate.
      [$tempStorage, $tempDir] = $this->createTempConfigDirectory();

      // Apply patch to temporary directory.
      $this->patchTool->applyMultiFileDiff($diff, $tempDir);

      // Create fresh storage object to detect files created by patch.
      $tempStorage = new FileStorage($tempDir);

      // Use StorageComparer and ConfigImporter for validation.
      $storageComparer = new StorageComparer($tempStorage, $this->activeStorage);
      $storageComparer->createChangelist();

      if (!$storageComparer->hasChanges()) {
        $errors[] = $this->t('No configuration changes detected after applying diff')->render();
      }
      else {
        // Validate with ConfigImporter.
        $configImporter = $this->productionConfigImporterService->createImporter(
          $tempStorage,
          $this->activeStorage
        );
        $configImporter->validate();

        // Count changes.
        foreach ($storageComparer->getAllCollectionNames() as $collection) {
          foreach (['create', 'update', 'delete', 'rename'] as $operation) {
            $changeCount += count($storageComparer->getChangelist($operation, $collection));
          }
        }
      }
    }
    catch (\Exception $e) {
      $errors[] = $e->getMessage();
      // Clean up on error.
      if ($tempDir) {
        $this->cleanupTempStorage($tempDir);
        $tempStorage = NULL;
        $tempDir = NULL;
      }
    }

    return [
      'valid' => empty($errors),
      'errors' => $errors,
      'config_count' => $changeCount,
      'temp_storage' => $tempStorage,
      'temp_dir' => $tempDir,
    ];
  }

}
