<?php

declare(strict_types=1);

namespace Drupal\Tests\config_preview_deploy\Functional;

use Drupal\Tests\BrowserTestBase;
use Drush\TestTraits\DrushTestTrait;

/**
 * Tests Drush commands for Config Preview Deploy.
 *
 * @group config_preview_deploy
 */
class DrushCommandsTest extends BrowserTestBase {

  use DrushTestTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'config_preview_deploy',
    'system',
    'user',
  ];

  /**
   * Tests cpd:init command.
   */
  public function testInitCommand(): void {
    $this->drush('config:preview-deploy:init');
    $this->assertStringContainsString('Base checkpoint created', $this->getOutput());
  }

  /**
   * Tests cpd:status command.
   */
  public function testStatusCommand(): void {
    // Initialize first.
    $this->drush('config:preview-deploy:init');

    $this->drush('config:preview-deploy:status');
    $output = $this->getOutput();
    $this->assertStringContainsString('Config Preview Deploy Status', $output);
    $this->assertStringContainsString('Environment:', $output);
    $this->assertStringContainsString('Base version:', $output);
  }

  /**
   * Tests cpd:diff command.
   */
  public function testDiffCommand(): void {
    // Initialize first.
    $this->drush('config:preview-deploy:init');

    // Should work even with no changes.
    $this->drush('config:preview-deploy:diff');
    $this->assertStringContainsString('No configuration changes detected', $this->getOutput());
  }

  /**
   * Tests cpd:rebase command is executable.
   */
  public function testRebaseCommandExecutable(): void {
    // Clear webhook configuration to ensure predictable behavior.
    $this->drush('config:set config_preview_deploy.settings webhook.enabled false');

    // Command should run without fatal errors (even if it fails logically).
    $this->drush('config:preview-deploy:rebase', [], [], NULL, NULL, 1);
    // Just verify the command executed.
    $this->assertTrue(TRUE, 'Rebase command executed without fatal errors');
  }

  /**
   * Tests cpd:apply command is executable.
   */
  public function testApplyCommandExecutable(): void {
    // Test the command without valid file to verify it's not fatally broken.
    // We expect this to fail but not with undefined method errors.
    $this->drush('config:preview-deploy:apply', [], [], NULL, NULL, 1);
    // Just verify the command executed without fatal translation errors.
    $this->assertTrue(TRUE, 'Apply command executed without fatal errors');
  }

  /**
   * Tests command aliases work.
   */
  public function testCommandAliases(): void {
    // Test short aliases.
    $this->drush('cpd:init');
    $this->assertStringContainsString('Base checkpoint created', $this->getOutput());

    $this->drush('cpd:status');
    $this->assertStringContainsString('Config Preview Deploy Status', $this->getOutput());

    $this->drush('cpd:diff');
    $this->assertStringContainsString('No configuration changes detected', $this->getOutput());
  }

  /**
   * Tests that commands work when patch tool is available.
   */
  public function testCommandsWithPatchTool(): void {
    // Skip if patch tool is not available.
    $patchTool = $this->container->get('config_preview_deploy.patch_tool');
    if (!$patchTool->isAvailable()) {
      $this->markTestSkipped('GNU patch tool is required for this test');
    }

    $this->drush('cpd:init');
    $this->assertStringContainsString('Base checkpoint created', $this->getOutput());

    $this->drush('cpd:status');
    $this->assertStringContainsString('Changes: None', $this->getOutput());
  }

  /**
   * Tests register environment command is executable.
   */
  public function testRegisterEnvironmentCommandExecutable(): void {
    // Test that the command exists and runs without fatal errors.
    $this->drush('config:preview-deploy:register-environment', ['https://test.example.com']);
    $output = $this->getOutput();

    // Command should execute successfully and register the environment.
    $this->assertStringContainsString('Registered preview environment', $output);
    $this->assertStringContainsString('https://test.example.com', $output);
  }

  /**
   * Tests register environment command alias works.
   */
  public function testRegisterEnvironmentCommandAlias(): void {
    // Test that the alias works.
    $this->drush('cpd:register', ['invalid-url'], [], NULL, NULL, 1);
    $output = $this->getOutput();

    // Should show URL validation error.
    $this->assertStringContainsString('Invalid base URI format', $output);
  }

}
