<?php

declare(strict_types=1);

namespace Drupal\Tests\config_preview_deploy\Kernel;

use Drupal\config_preview_deploy\ConfigVerifier;
use Drupal\KernelTests\KernelTestBase;
use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Psr7\Request;

/**
 * Tests the ConfigVerifier service.
 *
 * @group config_preview_deploy
 */
class ConfigVerifierTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['config_preview_deploy', 'system'];

  /**
   * The config verifier service.
   */
  protected ConfigVerifier $configVerifier;

  /**
   * Mock HTTP client.
   */
  protected Client $mockHttpClient;

  /**
   * Mock handler for HTTP requests.
   */
  protected MockHandler $mockHandler;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installConfig(['system', 'config_preview_deploy']);

    // Create mock HTTP client.
    $this->mockHandler = new MockHandler();
    $handlerStack = HandlerStack::create($this->mockHandler);
    $this->mockHttpClient = new Client(['handler' => $handlerStack]);

    // Create config verifier with mock HTTP client.
    $this->configVerifier = new ConfigVerifier(
      $this->mockHttpClient,
      $this->container->get('state'),
      $this->container->get('logger.factory'),
      $this->container->get('config.factory'),
      $this->container->get('date.formatter'),
      $this->container->get('tempstore.private'),
      $this->container->get('config_preview_deploy.hash_verification')
    );
  }

  /**
   * Tests verification request validation with various scenarios.
   */
  public function testVerificationRequestValidation(): void {
    // Mock HTTP_HOST for testing.
    $_SERVER['HTTP_HOST'] = 'localhost';
    $hashVerification = $this->container->get('config_preview_deploy.hash_verification');

    // Use current time and adjust for testing.
    $currentTime = time();

    // Test 1: Invalid verification (old timestamp).
    $this->container->get('state')->set('config_preview_deploy.last_change', $currentTime);

    // Older than last_change, so invalid.
    $baseTimestamp = $currentTime - 100;
    $authHash = $hashVerification->generateVerificationHash('localhost', $baseTimestamp);

    $request = [
      'environment' => 'pr-feature-123',
      'base_timestamp' => $baseTimestamp,
      'auth_hash' => $authHash,
    ];

    $result = $this->configVerifier->validateVerificationRequest($request);
    $this->assertIsArray($result);
    $this->assertArrayHasKey('valid', $result);
    $this->assertArrayHasKey('current_timestamp', $result);
    $this->assertArrayHasKey('last_deployed_from', $result);
    $this->assertFalse($result['valid'], 'Should be invalid since base is older');
    $this->assertEquals($currentTime, $result['current_timestamp']);

    // Test 2: Valid verification (new timestamp).
    $this->container->get('state')->set('config_preview_deploy.last_change', $currentTime - 200);

    // Newer than last_change, so valid.
    $baseTimestamp = $currentTime - 50;
    $authHash = $hashVerification->generateVerificationHash('localhost', $baseTimestamp);

    $request = [
      'environment' => 'pr-feature-123',
      'base_timestamp' => $baseTimestamp,
      'auth_hash' => $authHash,
    ];

    $result = $this->configVerifier->validateVerificationRequest($request);
    $this->assertTrue($result['valid'], 'Should be valid since base is newer');
    $this->assertEquals($currentTime - 200, $result['current_timestamp']);

    // Test 3: Authentication failure.
    $request = [
      'environment' => 'pr-feature-123',
      'base_timestamp' => $currentTime - 50,
      'auth_hash' => 'invalid-hash',
    ];

    $this->expectException(\RuntimeException::class);
    $this->expectExceptionMessage('Invalid authentication');
    $this->configVerifier->validateVerificationRequest($request);
  }

  /**
   * Tests updating last change timestamp.
   */
  public function testLastChangeTracking(): void {
    $environment = 'pr-feature-123';

    // Get the current timestamp (may already be set by event subscriber).
    $initialTimestamp = $this->container->get('state')->get('config_preview_deploy.last_change', 0);

    $this->configVerifier->updateLastChange($environment);

    // Verify timestamp was updated.
    $lastChange = $this->container->get('state')->get('config_preview_deploy.last_change');
    $this->assertGreaterThanOrEqual($initialTimestamp, $lastChange);
    $this->assertLessThanOrEqual(time(), $lastChange);

    // Verify environment was recorded.
    $this->assertEquals($environment, $this->container->get('state')->get('config_preview_deploy.last_deployed_from'));
  }

  /**
   * Tests rebase availability check.
   */
  public function testRebaseAvailabilityChecks(): void {
    // Initially not available.
    $this->assertFalse($this->configVerifier->isRebaseAvailable());

    // Enable webhook but no URL.
    $this->config('config_preview_deploy.settings')
      ->set('webhook.enabled', TRUE)
      ->save();
    $this->assertFalse($this->configVerifier->isRebaseAvailable());

    // Add URL.
    $this->config('config_preview_deploy.settings')
      ->set('webhook.url', 'https://webhook.example.com')
      ->save();
    $this->assertTrue($this->configVerifier->isRebaseAvailable());

    // Disable webhook.
    $this->config('config_preview_deploy.settings')
      ->set('webhook.enabled', FALSE)
      ->save();
    $this->assertFalse($this->configVerifier->isRebaseAvailable());
  }

  /**
   * Tests successful rebase trigger.
   */
  public function testTriggerRebaseSuccess(): void {
    // Configure webhook.
    $this->config('config_preview_deploy.settings')
      ->set('webhook.enabled', TRUE)
      ->set('webhook.url', 'https://webhook.example.com')
      ->save();

    // Mock successful response.
    $this->mockHandler->append(new Response(200, [], '{"status":"ok"}'));

    $result = $this->configVerifier->triggerRebase();
    $this->assertTrue($result);
  }

  /**
   * Tests rebase trigger error scenarios.
   */
  public function testRebaseTriggerErrors(): void {
    // Test 1: Rebase trigger failure when not configured.
    try {
      $this->configVerifier->triggerRebase();
      $this->fail('Expected RuntimeException was not thrown');
    }
    catch (\RuntimeException $e) {
      $this->assertStringContainsString('Rebase webhook is not configured', $e->getMessage());
    }

    // Test 2: Rebase trigger with HTTP error.
    $this->config('config_preview_deploy.settings')
      ->set('webhook.enabled', TRUE)
      ->set('webhook.url', 'https://webhook.example.com')
      ->save();

    // Mock HTTP error.
    $this->mockHandler->append(new RequestException(
      'Webhook failed',
      new Request('POST', 'https://webhook.example.com')
    ));

    try {
      $this->configVerifier->triggerRebase();
      $this->fail('Expected RuntimeException was not thrown');
    }
    catch (\RuntimeException $e) {
      $this->assertStringContainsString('Failed to trigger rebase webhook', $e->getMessage());
    }
  }

}
