<?php

declare(strict_types=1);

namespace Drupal\Tests\config_preview_deploy\Kernel;

use Drupal\config_preview_deploy\CheckpointDiff;
use Drupal\config_preview_deploy\Controller\PreviewController;
use Drupal\KernelTests\KernelTestBase;
use Drupal\user\Entity\Role;
use Drupal\user\Entity\User;

/**
 * Tests the PreviewController.
 *
 * @group config_preview_deploy
 */
class PreviewControllerTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'config_preview_deploy',
    'system',
    'user',
    'config',
  ];

  /**
   * The checkpoint diff service.
   */
  protected CheckpointDiff $checkpointDiff;

  /**
   * The preview controller.
   */
  protected PreviewController $controller;

  /**
   * Test user.
   */
  protected User $testUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installConfig(['system', 'user']);
    $this->installEntitySchema('user');

    // Create a test user with appropriate permissions.
    $role = Role::create([
      'id' => 'test_role',
      'label' => 'Test Role',
    ]);
    $role->grantPermission('deploy config from preview');
    $role->save();

    $this->testUser = User::create([
      'name' => 'testuser',
      'mail' => 'test@example.com',
      'roles' => ['test_role'],
    ]);
    $this->testUser->save();

    // Set up services.
    $this->checkpointDiff = $this->container->get('config_preview_deploy.checkpoint_diff');
    $this->controller = PreviewController::create($this->container);
  }

  /**
   * Tests the changes() method with various scenarios.
   */
  public function testChangesViewScenarios(): void {
    // Test 1: Changes view when no base checkpoint exists.
    $build = $this->controller->changes();
    $this->assertArrayHasKey('#title', $build);
    $this->assertEquals('Configuration Changes', $build['#title']->render());
    $this->assertArrayHasKey('no_changes', $build);
    $this->assertStringContainsString('No base checkpoint found', $build['no_changes']['#markup']);

    // Test 2: Changes view when no changes exist (but base checkpoint exists).
    $this->checkpointDiff->createBaseCheckpoint();
    $build = $this->controller->changes();
    $this->assertArrayHasKey('#title', $build);
    $this->assertEquals('Configuration Changes', $build['#title']->render());
    $this->assertArrayHasKey('no_changes', $build);
    $this->assertStringContainsString('No configuration changes detected', $build['no_changes']['#markup']);

    // Test 3: Changes view when changes exist.
    $this->config('system.site')
      ->set('name', 'Modified Site Name')
      ->save();

    $build = $this->controller->changes();
    $this->assertArrayHasKey('#title', $build);
    $this->assertEquals('Configuration Changes', $build['#title']->render());
    $this->assertArrayHasKey('changes_table', $build);
    $this->assertArrayHasKey('#attached', $build);
    $this->assertContains('core/drupal.dialog.ajax', $build['#attached']['library']);
    $this->assertArrayHasKey('download', $build, 'Download button should be present');
  }

  /**
   * Tests the diff() method with various scenarios.
   */
  public function testDiffViewScenarios(): void {
    // Test 1: Diff view with no base checkpoint.
    $build = $this->controller->diff('system.site');
    $this->assertArrayHasKey('#markup', $build);
    $this->assertStringContainsString('No base checkpoint found', $build['#markup']);

    // Test 2: Diff view with changes.
    $this->checkpointDiff->createBaseCheckpoint();
    $this->config('system.site')
      ->set('name', 'Modified Site Name')
      ->save();

    $build = $this->controller->diff('system.site');
    $this->assertArrayHasKey('#title', $build);
    $this->assertStringContainsString('system.site', $build['#title']->render());
    $this->assertArrayHasKey('diff', $build);
    $this->assertArrayHasKey('#attached', $build);
    $this->assertContains('system/diff', $build['#attached']['library']);
    $this->assertArrayHasKey('back', $build);
  }

  /**
   * Tests access control scenarios for changes route.
   */
  public function testAccessControlScenarios(): void {
    $access = $this->container->get('config_preview_deploy.preview_environment_access');

    // Test 1: Access control allowed in preview environment.
    $this->container->get('current_user')->setAccount($this->testUser);
    $result = $access->changesAccess($this->testUser);
    $this->assertTrue($result->isAllowed(), 'Changes access should be allowed in preview environment');

    // Test 2: Access forbidden on production.
    putenv('DRUPAL_CONFIG_PREVIEW_DEPLOY_IS_PRODUCTION=1');
    $result = $access->changesAccess($this->testUser);
    $this->assertTrue($result->isForbidden(), 'Changes access should be forbidden in production environment');

    // Clean up.
    putenv('DRUPAL_CONFIG_PREVIEW_DEPLOY_IS_PRODUCTION');
  }

  /**
   * Tests dashboard functionality and change count display.
   */
  public function testDashboardFunctionality(): void {
    // Create a base checkpoint first.
    $this->checkpointDiff->createBaseCheckpoint();

    // Modify some configuration.
    $this->config('system.site')
      ->set('name', 'Modified Site Name')
      ->save();

    $build = $this->controller->dashboard();

    $this->assertArrayHasKey('environment_info', $build);
    $this->assertArrayHasKey('actions', $build);

    // Check that environment info contains change count.
    $envItems = $build['environment_info']['details']['#items'];
    $hasChangeCount = FALSE;
    foreach ($envItems as $item) {
      if (strpos($item->render(), 'Pending changes:') !== FALSE) {
        $hasChangeCount = TRUE;
        break;
      }
    }
    $this->assertTrue($hasChangeCount, 'Environment info should contain pending changes count');
  }

  /**
   * Tests download diff functionality.
   */
  public function testDownloadDiff(): void {
    // Create a base checkpoint and make changes.
    $this->checkpointDiff->createBaseCheckpoint();
    $this->config('system.site')
      ->set('name', 'Modified Site Name')
      ->save();

    // Test that download functionality is available.
    $response = $this->controller->downloadDiff();
    $this->assertInstanceOf('Symfony\Component\HttpFoundation\Response', $response);
    $this->assertEquals(200, $response->getStatusCode());
    $this->assertStringContainsString('text/plain', $response->headers->get('Content-Type'));
  }

}
